#!/usr/bin/env python3
from numba import jit

def numba_integrate(f, a, b, N):
    f_jit = jit("f8(f8)", nopython=True)(f)

    @jit("f8(f8)", nopython=True)
    def f_call(x):
        return f_jit(x)
    
    @jit("f8(f8, f8, u8)", nopython=True)
    def numba_integrate_call(a, b, N):
        width = float((b-a)/N)
        result=0.0
        i = 1
        while(i < N):
            xi = float(a + (i * width))
            result += float(f_call(xi))
            i += 1
        result = width * (result + float(f_call(b)))
        return result
    return numba_integrate_call(a, b, N)

def numba_midpoint_integrate(f, a, b, N):
    f_jit = jit("f8(f8)", nopython=True)(f)

    @jit("f8(f8)", nopython=True)
    def f_call(x):
        return f_jit(x)

    @jit("f8(f8, f8, u8)", nopython=True)
    def numba_integrate_call(a, b, N):
        width = float((b-a)/N)
        result=0.0
        i = 1
        while(i < N):
            xi = float((2*a + ((2*i - 1) * width))/2.0)
            result += float(f_call(xi))
            i += 1
        result = width * (result + float(f_call((((a+((N-1) * width)) + b))/2.0)))
        return result
    return numba_integrate_call(a, b, N)



#!/usr/bin/env python3
import numpy
cimport numpy

cpdef double cython_integrate(f, a, b, N):
    cdef double width = (b-a)/N
    cdef numpy.ndarray[numpy.double_t, ndim=1] points = numpy.linspace(a+width, b, N, dtype=numpy.double)
    f = numpy.vectorize(f)
    cdef double result = numpy.sum(f(points), dtype=numpy.double) * width
    return result

cpdef double cython_midpoint_integrate(f, a, b, N):
    cdef double width = (b-a)/N
    cdef numpy.ndarray[numpy.double_t, ndim=1] points = numpy.linspace((2*a+width)/2.0, (((a+((N-1) * width)) + b)/2.0), N, dtype=numpy.double)
    f = numpy.vectorize(f)
    cdef double result = numpy.sum(f(points), dtype=numpy.double) * width
    return result


#!/usr/bin/env python3
import numpy

def numpy_integrate(f, a, b, N):
    width = float((b-a)/N)
    points = numpy.linspace(a+width, b, N)
    f = numpy.vectorize(f)
    result = numpy.sum(f(points)) * width
    return result

def numpy_midpoint_integrate(f, a, b, N):
    width = float((b-a)/N)
    points = numpy.linspace((2*a+width)/2.0, (((a+((N-1) * width)) + b)/2.0), N)
    f = numpy.vectorize(f)
    result = numpy.sum(f(points)) * width
    return result


#!/usr/bin/env python3
from integrator import python_integrate, python_midpoint_integrate
from numpy_integrator import numpy_integrate, numpy_midpoint_integrate
from numba_integrator import numba_integrate, numba_midpoint_integrate
from cython_integrator import cython_integrate, cython_midpoint_integrate

# f --> function to integrate
# a --> lower limit
# b --> upper limit
# N --> number of intermediate points
# method --> "endpoint" or "midpoint"
# implementation --> "python" or "numpy" or "numba" or "cython" 
def integrate(f, a, b, N, method = "endpoint", implementation = "python"):
    if(implementation == "numpy"):
        if(method == "midpoint"):
            return numpy_midpoint_integrate(f, a, b, N)
        else: # endpoint
            return numpy_integrate(f, a, b, N)

    elif(implementation == "numba"):
        if(method == "midpoint"):
            return numba_midpoint_integrate(f, a, b, N)
        else: # endpoint
            return numba_integrate(f, a, b, N)

    elif(implementation == "cython"):
        if(method == "midpoint"):
            return cython_midpoint_integrate(f, a, b, N)
        else: # endpoint
            return cython_integrate(f, a, b, N)

    else: #python
        if(method == "midpoint"):
            return python_midpoint_integrate(f, a, b, N)
        else: # endpoint
            return python_integrate(f, a, b, N)


#!/usr/bin/env python3

def integrate(f, a, b, N):
    width = float((b-a)/N)
    result=0.0
    i = 1
    while(i < N):
        xi = float(a + (i * width))
        result += float(f(xi))
        i += 1
    result = width * (result + float(f(b)))
    return result

def midpoint_integrate(f, a, b, N):
    width = float((b-a)/N)
    result=0.0
    i = 1
    while(i < N):
        xi = float((2*a + ((2*i - 1) * width))/2.0)
        result += float(f(xi))
        i += 1
    result = width * (result + float(f((((a+((N-1) * width)) + b))/2.0)))
    return result


#!/usr/bin/env python3
from distutils.core import setup
from Cython.Build import cythonize
import numpy

setup(
    name = "Integration",
    ext_modules = cythonize("*.pyx"),
    include_dirs=[numpy.get_include()]
)


#!/usr/bin/env python3
from distutils.core import setup
from distutils.extension import Extension
import numpy

try:
    from Cython.Distutils import build_ext
except ImportError:
    raise ImportError("could not import cython")

cmdclass = {}
ext_modules = []

ext_modules += [
        Extension("integrator_mandubab.cython_integrator", [ "integrator_mandubab/cython_integrator.pyx" ],
        include_dirs=[numpy.get_include()]),
    ]
cmdclass.update({ 'build_ext': build_ext })

setup(
    name='integrator_mandubab',
    version='0.2',
    description='Simple Integrator for INF4331 assignment4 in UiO',
    url='https://github.com/UiO-INF3331/INF3331-mandubab/tree/master/assignment4',
    author='Manduba Annka Bari',
    author_email='m.a.bari@sfe.uio.no',
    packages=[ 'integrator_mandubab'],
    cmdclass = cmdclass,
    ext_modules=ext_modules,
    long_description=open('README.txt').read(),
    license="MIT",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Cython',
        'Topic :: Scientific/Engineering :: Mathematics',
    ],
    keywords='simple integrator UiO INF4331 assignment4',
)


#!/usr/bin/env python3
from integrator import integrate, midpoint_integrate
from numpy_integrator import numpy_integrate, numpy_midpoint_integrate
from numba_integrator import numba_integrate, numba_midpoint_integrate
from cython_integrator import cython_integrate, cython_midpoint_integrate

def test_integral_of_constant_function(f, N):
    print("TESTING: integration of constant function using ", f.__name__, " and N=", N)
    assert abs(f(lambda x: 2, 0, 1, N) - 2) < 10**-10 
    print("integration of constant function using ", f.__name__, " and N=", N, "test: PASSED")

def test_integral_of_linear_function(f, N):
    print("TESTING: integration of linear function using ", f.__name__, " and N=", N)
    assert abs(f(lambda x: 2*x, 0, 1, N) - 1) <= (1.0/N + 10**-10)
    print("integration of linear function using ", f.__name__, " and N=", N, "test: PASSED")
    
#for N in range(1, 100002, 100):
#    test_integral_of_constant_function(integrate, N)
#    test_integral_of_constant_function(numpy_integrate, N)
#    test_integral_of_constant_function(numba_integrate, N)
#    test_integral_of_constant_function(cython_integrate, N)

#    test_integral_of_constant_function(midpoint_integrate, N)
#    test_integral_of_constant_function(numpy_midpoint_integrate, N)
#    test_integral_of_constant_function(numba_midpoint_integrate, N)
#    test_integral_of_constant_function(cython_midpoint_integrate, N)



